#!/bin/bash
#set -n		# read but do not execute commands (check on syntax errors)
#
# File: /usr/local/bin/killproc.sh
#
# Send signals to (shell) processes by pathname.
#
#    Copyright (C) 2017  Juergen Kaesmann (JK)

##############################################################################
# GPL NOTICE

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

##############################################################################
# EXIT CODES
# Standard

##############################################################################
# SOURCE FILES

. /usr/local/etc/main.conf				# config main
. $LLIBDIR/libfuncstd-sh				# functions standard
#. /usr/local/share/ksm-gpgswitch/libfuncstd-sh
#. /usr/local/etc/sample.conf				# config sample
#. ~/.samplerc						# config user

##############################################################################
# DECLARATION OF CONSTANTS AND VARIABLES

#------------------STANDARD CONSTANTS AND VARIABLES---------------------------
readonly package=Root-Admin				# package name
readonly vinfopath='/usr/local/share/ksm-root-admin'	# path to package dir
readonly version=$(/bin/cat $vinfopath/VERSION)		# package version
readonly release=$(/bin/cat $vinfopath/RELEASE)		# package release
readonly scriptname=$(basename $0)			# name of script
readonly tempfile=$TEMPDIR/$scriptname.$UID.tmp		# temporary file name
#tempfile=$(mktemp $TEMPDIR/$scriptname.$UID.XXXXXX)	# temporary file name
readonly debuglog="/bin/logger -p user.debug -t $scriptname -- DEBUG:"	# log dbg msg
readonly errlog="/bin/logger -p user.err -t $scriptname -- ERROR:"	# log err msg
readonly infolog="/bin/logger -p user.info -t $scriptname -- INFO:"	# log inf msg
cmd=''							# sub command
debug=no						# debug=yes/no
verbose=''						# verbose=yes/''/no

#------------------USER FILL IN AFTER INSTALLATION----------------------------
#echo "$scriptname line 59: Setup following 2 vars. Then comment out this line." >&2; exit 1
#readonly var1='Dummy1'
#var2='Dummy2'

#-----------------------USER-CONSTANTS----------------------------------------
# STRINGS
#readonly const=string

# NUMBERS
#readonly const=value

#-----------------------USER-VARIABLES----------------------------------------
# STRINGS
#args=''				# all args from cmdline
arg1=''					# 1. arg from cmdline
#opt_i=''				# option -i (arg)
#opt_o=''				# option -o (yes)
#sample-opt=''				# option --sample-opt (yes/no/arg)
#infile=''				# option --infile (arg)
#outfile=''				# option --outfile (arg)
opt_p=''				# option -p (arg)
opt_s=''				# option -s (arg)

# NUMBERS
err=0					# error code

# ARRAYS
# Indexed arrays are always zero based string arrays.
# If index starts with 1 and all members are assigned then
# count of members of array = last index of array
#declare -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array

##############################################################################
# SIGNAL HANDLING

##############################################################################
# FUNCTIONS

#=============================================================================
function helptext ()
{
# Output helptext
# Uses cat, echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG helptext: $# $@" >&2
	$debuglog helptext: $# $@
fi

cat <<EOT
NAME
$scriptname - Send signals to (shell) processes by pathname.

SYNOPSIS
$scriptname [options] [process]

DESCRIPTION
Send signals to (shell) processes by pathname.

1. Get PID2 from PID-File (if any).
2. Then retrieve PID from process with pathname.
3. Then check PID and PID2:

   If no PID2 found (no PID-File) then
     assume proc is dead.

   If no PID-File found then
     send signal to PID2 and remove PID-File if signal was TERM/KILL.

   If both PIDs are equal then
     send signal to PID and remove PID-File if signal was TERM/KILL.

   If they differ then
     send signal to PID.

If option -s not given then the signal is SIGTERM.
If SIGTERM does not work then send the signal SIGKILL after 5 seconds.

Defaults:  Send signals to processes.

OPTIONS
Gnu style options:

  -h|--help    = Output help, then exit.
  -V|--version = Output version info, then exit.
  -d|--debug   = Turn on DEBUG mode.
  -q|--quiet   = Be quiet.
  -v|--verbose = Be verbose.
  -p <pidfile> = Pidfile to use. Send the signal to the PID read from
                 this file. If the pidfile does not exist, $scriptname
                 assumes that the daemon is not running.
  -s <SIG>     = Send signal <SIG>.
  --           = End of options.

ARGUMENTS
Give if needed:

  process = Pathname of process to kill

EXIT STATUS
On program exit:

  Return values acc. to LSB for all commands but status:
  0	  - success
  1       - generic or unspecified error
  2       - invalid or excess argument(s)
  3       - unimplemented feature (e.g. "reload")
  4       - user had insufficient privileges
  5       - program is not installed
  6       - program is not configured
  7       - program is not running
  8--199  - reserved (8--99 LSB, 100--149 distrib, 150--199 appl)
  
  Note that starting an already running service, stopping
  or restarting a not-running service as well as the restart
  with force-reload (in case signaling is not supported) are
  considered a success.

FILES
Used directories and files:

  Default Pid file = /var/run/<basename arg1>.pid

SEE ALSO
Related manpages:

  showerrmsg(1).

AUTHORS
Juergen Kaesmann <juergen@kaesmann-online.de>
EOT
#  -o|--output  = Output goes to STDOUT.
#  -i f         = Input from file <f>.
#  --infile=f   = Input from file <f>.

#EXAMPLES
#For instance:
#
#  Do it:
#  $scriptname

return
}

#=============================================================================
function send_sig_kill ()
{
# Send signals to proc with PID.
# par1 = pid
# Uses echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG send_sig_kill: $# $@" >&2
	$debuglog send_sig_kill: $# $@
fi

local pid="$1" ret=0

# send signal term/kill to proc pid
test "$verbose" == yes && echo "killproc send_sig_kill: Try to TERM PID $pid"
kill -s TERM $pid
sleep 5

# look if pid from ps has been killed
/bin/ps -p $pid -o pid= > /dev/null 2>&1
ret=$?
if [ "$ret" -eq 0 ]; then # proc not term.
	test "$verbose" == yes && echo "killproc send_sig_kill: Try to KILL PID $pid"
	kill -s KILL $pid
fi
ret=0

return $ret
}

#=============================================================================
function mainproc ()
{
# Sample function.
# par1 = 
# Uses cat, echo, logger, ps, rm
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG mainproc: $# $@" >&2
	$debuglog mainproc: $# $@
fi

local b='' pidfile='' pidfound=no pid2found=no
local pid=0 pid2=0 ret=0
#local -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array
#s=$(date "+%d.%m.%Y %H:%M:%S")		# DD.MM.YYYY  HH:MM:SS

test "$verbose" == yes && echo -e "\nkillproc mainproc: Argument=$arg1"
if [ -e "$arg1" ]; then
	# setup pidfile
	b=$(basename $arg1)
	if [ -z "$opt_p" ]; then
		pidfile=/var/run/$b.pid
	else
		pidfile="$opt_p"
	fi
	test "$verbose" == yes && echo "killproc mainproc: Using pidfile=$pidfile"

	# get pid2 from pidfile if any
	test -e $pidfile && pid2=$(/bin/cat $pidfile)
	ret=$?
	if [ "$ret" -eq 0 ]; then			# pid from pidfile found
		test "$verbose" == yes && echo "killproc mainproc: PID2=$pid2 from pidfile"
		pid2found=yes
	fi

	# get pid from ps
	Get_PID $arg1 pid
	ret=$?
	if [ "$ret" -eq 0 ]; then			# pid from pidfile found
		test "$verbose" == yes && echo "killproc mainproc: PID $pid with ps from $arg1"
		pidfound=yes
	fi

	if [ "$pidfound" == yes ] || [ "$pid2found" == yes ]; then # pid or pid2 found
		if [ "$pid" == "$pid2" ]; then		# both are equal
			if [ -z "$opt_s" ]; then	# no opt_s
				send_sig_kill "$pid2"

				# rm pidfile
				test "$verbose" == yes && echo "killproc mainproc: pidfile $pidfile will be removed"
				/bin/rm -f $pidfile
			else				# opt_s given
				# send signal opt_s to proc pid2
				test "$verbose" == yes && echo "killproc mainproc: send signal $opt_s to PID2 $pid2"
				kill -s $opt_s $pid2
			fi
			ret=0
		elif [ "$pidfound" == yes ] && [ "$pid2found" == no ]; then # only pid exist
			# assume no proc
			test "$verbose" == yes && echo "killproc mainproc: no pidfile - assume proc is dead"
			ret=0
		elif [ "$pidfound" == no ] && [ "$pid2found" == yes ]; then
			if [ -z "$opt_s" ]; then	# no opt_s
				test "$verbose" == yes && echo "killproc mainproc: send TERM/KILL to PID2 $pid2"
				send_sig_kill "$pid2"

				# rm pidfile
				test "$verbose" == yes && echo "killproc mainproc: pidfile $pidfile will be removed"
				/bin/rm -f $pidfile
			else				# opt_s given
				# send signal opt_s to proc pid2
				test "$verbose" == yes && echo "killproc mainproc: send signal $opt_s to PID2 $pid2"
				kill -s $opt_s $pid2
			fi
			ret=0
		else					# pid <> pid2
			if [ -z "$opt_s" ]; then	# no opt_s
				test "$verbose" == yes && echo "killproc mainproc: send TERM/KILL to PID $pid"
				send_sig_kill "$pid"

				test "$verbose" == yes && echo "killproc mainproc: do not touch pidfile $pidfile"
				# rm pidfile
				#test "$verbose" == yes && echo "killproc mainproc: remove $pidfile"
				#/bin/rm $pidfile
			else				# opt_s given
				# send signal opt_s to proc pid2
				test "$verbose" == yes && echo "killproc mainproc: send signal $opt_s to PID2 $pid2"
				kill -s $opt_s $pid2
			fi
			ret=0
		fi
	else						# no pid nor pid2 found
		test "$verbose" == yes && echo "killproc mainproc: program is not running"
		ret=0
	fi
else							# program not found
	test "$verbose" == yes && echo "killproc mainproc: program $arg1 not installed"
	ret=5
fi

return $ret
}

##############################################################################
# PARSE COMMANDLINE

# Try to read all options with args and all remaining args into variables.
# Stop parsing when all options are done. Try to do a check on all options,
# parameters and version numbers. Exit on error.

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG Parse Commandline: $# $@" >&2
	$debuglog Parse Commandline: $# $@
fi

##############################################################################
# GET OPTIONS
# Give 3 params to function GetOpt: Posixoptions Gnuoptions cmdline-args
# Samples: 'hVdqvi:' 'help version debug quiet verbose infile=' $@
# A ':' after an POSIX option means this option needs an arg.
# A '=' after a Gnu long option means this option needs an arg.
# When GetOpt finishes then rm all options from the cmdline.
# echo "DEBUG GET OPTIONS: _OPTNAME_=$_OPTNAME_  _OPTVAL_=$_OPTVAL_  _OPTIDX_=$_OPTIDX_  _OPTPOS_=$_OPTPOS_" >&2

err=0
while [ "$err" -eq 0 ]; do
	GetOpt 'hVdqvp:s:' 'help version debug quiet verbose' $@
	err=$?
	if [ $err -eq 0 ]; then
		case "$_OPTNAME_" in
		    h|help)	cmd="-h";;
		    V|version)	cmd="-V";;
#		    a)		cmd="-${_OPTNAME_}";;
		    d|debug)	debug=yes;;
		    q|quiet)	verbose=no;;
		    v|verbose)	verbose=yes;;
#		    o|output)	opt_o=yes;;
#		    i|infile)	opt_i="$_OPTVAL_";;
		    p)		opt_p="$_OPTVAL_";;
		    s)		opt_s="$_OPTVAL_";;
		    --)		break;;
		    :)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[8]} -${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[8]} -${_OPTVAL_}
			fi; exit 3;;
		    ?)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[4]} -${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[4]} -${_OPTVAL_}
			fi; exit 3;;
		    *)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[99]} _OPTNAME_=${_OPTNAME_} _OPTVAL_=${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[99]} _OPTNAME_=${_OPTNAME_} _OPTVAL_=${_OPTVAL_}
			fi; exit 3;;
		esac
	fi
done
err=0
#if [ -z "$cmd" ]; then					# error no cmd given
#	if [ "$verbose" != no ]; then
#		echo "${ErrMsg[7]}" >&2
#	else
#		$errlog ${ErrMsg[7]}
#	fi
#	exit 7
#fi
shift ${_OPTIDX_}				# rm all opts from cmdline

##############################################################################
# CHECK OPTIONS, ARGUMENTS AND VERSION

# check not allowed options for sub commands
#case "$cmd" in
#    -l)						# for these cmds
#	if [ -n "$opt_z" ]; then			# option -z not allowed
#		if [ "$verbose" != no ]; then
#			echo "${ErrMsg[13]} -z" >&2
#		else
#			$errlog ${ErrMsg[13]} -z
#		fi
#		exit 13
#	fi
#	;;
#    *)							# for all other cmds
#	:
#	;;
#esac

# check needed options for sub commands
#case "$cmd" in
#    -l)						# for these cmds
#	if [ -z "$opt_z" ]; then			# option -z needed
#		if [ "$verbose" != no ]; then
#			echo "${ErrMsg[8]} -z" >&2
#		else
#			$errlog ${ErrMsg[8]} -z
#		fi
#		exit 8
#	fi
#	;;
#    *)							# for all other cmds
#	:
#	;;
#esac

# check options which are mutually exclusive
#if [ -n "$opt_x" ] && [ -n "$opt_y" ]; then		# don't give both opts
#		if [ "$verbose" != no ]; then
#			echo "${ErrMsg[13]} ${opt_x} and ${opt_y}" >&2
#		else
#			$errlog ${ErrMsg[13]} ${opt_x} and ${opt_y}
#		fi
#		exit 13
#fi

# get value from arg of option -i to var 'infile'
#if [ -n "$opt_i" ]; then
#	infile="$opt_i"
#fi

# check arguments after sub commands and options
case "$cmd" in
    '')
	if [ "$#" -eq 1 ]; then				# parameter count ok
		arg1="$1"
		shift
		#args="$@"
		#shift $#
	else						# error in param count
		if [ "$verbose" != no ]; then
			echo "${ErrMsg[2]}" >&2
		else
			$errlog ${ErrMsg[2]}
		fi
		exit 3
	fi
	;;
    *)
	if [ "$#" -ne 0 ]; then				# error in param count
		if [ "$verbose" != no ]; then
			echo "${ErrMsg[2]}" >&2
		else
			$errlog ${ErrMsg[2]}
		fi
		exit 3
	fi
	;;
esac

# check version of config files. Conf file = global, Rc file = user specific
#if [ "$version" != "$Sample_Conf" ] || [ "$version" != "$Sample_Rc" ]; then
#	if [ "$verbose" != no ]; then
#		echo "${ErrMsg[12]}" >&2
#	else
#		$errlog ${ErrMsg[12]}
#	fi
#	exit 12
#fi

##############################################################################
# MAINPROGRAM

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG Main: $# $@" >&2
	$debuglog Main: $# $@
fi

# Check what to do (sub commands, options and args are removed from cmdline)
case "$cmd" in
    '')							# no cmd given
	mainproc
	err=$?
	;;
    -h)							# cmd -h
	helptext
	err=0
	;;
    -V)							# cmd -V
	echo "$scriptname (${package}) ${version}-${release}"
	err=0
	;;
    *)							# other cmds = error
	if [ "$verbose" != no ]; then
		echo "${ErrMsg[6]}" >&2
	else
		$errlog ${ErrMsg[6]}
	fi
	err=1
	;;
esac

exit $err
